@@ -8,16 +8,15 @@
 
 #include "nmcli.h"
 
-NMCResultCode do_connections (NmCli *nmc, int argc, char **argv);
-
 void monitor_connections (NmCli *nmc);
 
 gboolean
-nmc_read_connection_properties (NmCli *nmc,
-                                NMConnection *connection,
-                                int *argc,
-                                char ***argv,
-                                GError **error);
+nmc_process_connection_properties (NmCli *nmc,
+                                   NMConnection *connection,
+                                   int *argc,
+                                   const char *const**argv,
+                                   gboolean allow_remove_setting,
+                                   GError **error);
 
 NMMetaColor nmc_active_connection_state_to_color (NMActiveConnectionState state);
 
