@@ -1,7 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0+
-/* nmcli - command-line tool to control NetworkManager
- *
- * (C) Copyright 2010 - 2018 Red Hat, Inc.
+/*
+ * Copyright (C) 2010 - 2018 Red Hat, Inc.
  */
 
 #ifndef NMC_DEVICES_H
@@ -9,8 +8,6 @@
 
 #include "nmcli.h"
 
-NMCResultCode do_devices (NmCli *nmc, int argc, char **argv);
-
 void nmc_complete_device (NMClient *client, const char *prefix, gboolean wifi_only);
 
 void nmc_complete_bssid (NMClient *client, const char *ifname, const char *bssid_prefix);
