@@ -1032,22 +1032,50 @@ verify (NMSetting *setting, NMConnection *connection, GError **error)
 
 	if (priv->interface_name) {
 		GError *tmp_error = NULL;
-		gboolean valid_ifname = FALSE;
+		NMUtilsIfaceType iface_type;
 
-		/* do not perform a interface name length check for OVS connection types
-		 * as they don't have a corresponding kernel link that enforces the 15 bytes limit.
-		 * Here we're whitelisting the OVS interface type as well, even if most OVS
-		 * iface types do have the limit, to let the OVS specific nm-setting verify whether the iface name
-		 * is good or not according to the internal type (internal, patch, ...) */
 		if (NM_IN_STRSET (type,
 		                  NM_SETTING_OVS_BRIDGE_SETTING_NAME,
-		                  NM_SETTING_OVS_PORT_SETTING_NAME,
-		                  NM_SETTING_OVS_INTERFACE_SETTING_NAME))
-			valid_ifname = nm_utils_ifname_valid (priv->interface_name, NMU_IFACE_OVS, &tmp_error);
-		else
-			valid_ifname = nm_utils_ifname_valid (priv->interface_name, NMU_IFACE_KERNEL, &tmp_error);
+		                  NM_SETTING_OVS_PORT_SETTING_NAME))
+			iface_type = NMU_IFACE_OVS;
+		else if (nm_streq (type, NM_SETTING_OVS_INTERFACE_SETTING_NAME)) {
+			NMSettingOvsInterface *s_ovs_iface = NULL;
+			const char *ovs_iface_type;
 
-		if (!valid_ifname) {
+			if (connection)
+				s_ovs_iface = nm_connection_get_setting_ovs_interface (connection);
+			_nm_setting_ovs_interface_verify_interface_type (s_ovs_iface,
+			                                                 s_ovs_iface ? nm_setting_ovs_interface_get_interface_type (s_ovs_iface) : NULL,
+			                                                 connection,
+			                                                 FALSE,
+			                                                 NULL,
+			                                                 &ovs_iface_type,
+			                                                 NULL);
+			if (!ovs_iface_type) {
+				/* We cannot determine to OVS interface type. Consequently, we cannot
+				 * fully validate the interface name.
+				 *
+				 * If we have a connection (and we do a full validation anyway), skip the
+				 * check. The connection will fail validation when we validate the OVS setting.
+				 *
+				 * Otherwise, do the most basic validation.
+				 */
+				if (connection)
+					goto after_interface_name;
+				iface_type = NMU_IFACE_ANY;
+			} else if (NM_IN_STRSET (ovs_iface_type, "patch")) {
+				/* this interface type is internal to OVS. */
+				iface_type = NMU_IFACE_OVS;
+			} else {
+				/* This interface type also requires a netdev. We need to validate
+				 * for both OVS and KERNEL. */
+				nm_assert (NM_IN_STRSET (ovs_iface_type, "internal", "system", "dpdk"));
+				iface_type = NMU_IFACE_OVS_AND_KERNEL;
+			}
+		} else
+			iface_type = NMU_IFACE_KERNEL;
+
+		if (!nm_utils_ifname_valid (priv->interface_name, iface_type, &tmp_error)) {
 			g_set_error (error,
 			             NM_CONNECTION_ERROR,
 			             NM_CONNECTION_ERROR_INVALID_PROPERTY,
@@ -1057,6 +1085,7 @@ verify (NMSetting *setting, NMConnection *connection, GError **error)
 			return FALSE;
 		}
 	}
+after_interface_name:
 
 	is_slave = FALSE;
 	slave_setting_type = NULL;
@@ -1251,11 +1280,14 @@ verify (NMSetting *setting, NMConnection *connection, GError **error)
 }
 
 static const char *
-find_virtual_interface_name (GVariant *connection_dict)
+find_virtual_interface_name (GVariant *connection_dict,
+                             GVariant **variant_to_free)
 {
 	GVariant *setting_dict;
 	const char *interface_name;
 
+	nm_assert (variant_to_free && !*variant_to_free);
+
 	setting_dict = g_variant_lookup_value (connection_dict, NM_SETTING_BOND_SETTING_NAME, NM_VARIANT_TYPE_SETTING);
 	if (!setting_dict)
 		setting_dict = g_variant_lookup_value (connection_dict, NM_SETTING_BRIDGE_SETTING_NAME, NM_VARIANT_TYPE_SETTING);
@@ -1267,39 +1299,15 @@ find_virtual_interface_name (GVariant *connection_dict)
 	if (!setting_dict)
 		return NULL;
 
+	*variant_to_free = setting_dict;
+
 	/* All of the deprecated virtual interface name properties were named "interface-name". */
 	if (!g_variant_lookup (setting_dict, "interface-name", "&s", &interface_name))
 		interface_name = NULL;
 
-	g_variant_unref (setting_dict);
 	return interface_name;
 }
 
-static gboolean
-nm_setting_connection_set_interface_name (NMSetting *setting,
-                                          GVariant *connection_dict,
-                                          const char *property,
-                                          GVariant *value,
-                                          NMSettingParseFlags parse_flags,
-                                          GError **error)
-{
-	const char *interface_name;
-
-	/* For compatibility reasons, if there is an invalid virtual interface name,
-	 * we need to make verification fail, even if that virtual name would be
-	 * overridden by a valid connection.interface-name.
-	 */
-	interface_name = find_virtual_interface_name (connection_dict);
-	if (!interface_name || nm_utils_ifname_valid_kernel (interface_name, NULL))
-		interface_name = g_variant_get_string (value, NULL);
-
-	g_object_set (G_OBJECT (setting),
-	              NM_SETTING_CONNECTION_INTERFACE_NAME, interface_name,
-	              NULL);
-
-	return TRUE;
-}
-
 static gboolean
 nm_setting_connection_no_interface_name (NMSetting *setting,
                                          GVariant *connection_dict,
@@ -1308,8 +1316,9 @@ nm_setting_connection_no_interface_name (NMSetting *setting,
                                          GError **error)
 {
 	const char *virtual_interface_name;
+	gs_unref_variant GVariant *variant_to_free = NULL;
 
-	virtual_interface_name = find_virtual_interface_name (connection_dict);
+	virtual_interface_name = find_virtual_interface_name (connection_dict, &variant_to_free);
 	g_object_set (G_OBJECT (setting),
 	              NM_SETTING_CONNECTION_INTERFACE_NAME, virtual_interface_name,
 	              NULL);
@@ -1759,7 +1768,6 @@ nm_setting_connection_class_init (NMSettingConnectionClass *klass)
 	                              obj_properties[PROP_INTERFACE_NAME],
 	                              NM_SETT_INFO_PROPERT_TYPE (
 	                                  .dbus_type             = G_VARIANT_TYPE_STRING,
-	                                  .from_dbus_fcn         = nm_setting_connection_set_interface_name,
 	                                  .missing_from_dbus_fcn = nm_setting_connection_no_interface_name,
 	                              ));
 
