@@ -27,7 +27,7 @@ NM_GOBJECT_PROPERTIES_DEFINE (NMSettingSriov,
 /**
  * NMSettingSriov:
  *
- * SR-IOV settings.
+ * SR-IOV settings
  *
  * Since: 1.14
  */
@@ -234,7 +234,7 @@ vf_add_vlan (NMSriovVF *vf,
 		vf->vlans = _vf_vlan_create_hash ();
 
 	g_hash_table_add (vf->vlans, vlan);
-	g_clear_pointer (&vf->vlan_ids, g_free);
+	nm_clear_g_free (&vf->vlan_ids);
 }
 
 /**
@@ -530,7 +530,7 @@ nm_sriov_vf_remove_vlan (NMSriovVF *vf, guint vlan_id)
 	    || !g_hash_table_remove (vf->vlans, &vlan_id))
 		return FALSE;
 
-	g_clear_pointer (&vf->vlan_ids, g_free);
+	nm_clear_g_free (&vf->vlan_ids);
 	return TRUE;
 }
 
@@ -1267,7 +1267,8 @@ nm_setting_sriov_class_init (NMSettingSriovClass *klass)
 	 *
 	 * Note that when the sriov setting is present NetworkManager
 	 * enforces the number of virtual functions on the interface
-	 * also when it is zero. To prevent any changes to SR-IOV
+	 * (also when it is zero) during activation and resets it
+	 * upon deactivation. To prevent any changes to SR-IOV
 	 * parameters don't add a sriov setting to the connection.
 	 *
 	 * Since: 1.14
