@@ -229,6 +229,20 @@
 # define NM_AVAILABLE_IN_1_24
 #endif
 
+#if NM_VERSION_MIN_REQUIRED >= NM_VERSION_1_26
+# define NM_DEPRECATED_IN_1_26           G_DEPRECATED
+# define NM_DEPRECATED_IN_1_26_FOR(f)    G_DEPRECATED_FOR(f)
+#else
+# define NM_DEPRECATED_IN_1_26
+# define NM_DEPRECATED_IN_1_26_FOR(f)
+#endif
+
+#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_26
+# define NM_AVAILABLE_IN_1_26            G_UNAVAILABLE(1,26)
+#else
+# define NM_AVAILABLE_IN_1_26
+#endif
+
 /*
  * Synchronous API for calling D-Bus in libnm is deprecated. See
  * https://developer.gnome.org/libnm/stable/usage.html#sync-api
@@ -249,10 +263,11 @@
  *     around g_dbus_connection_call_sync(). You may call it directly
  *     without feeling dirty.
  *
- * We don't want to force users away from this API, for that reason the
- * macro does not yet expand to G_DEPRECATED.
+ * The API is marked as deprecated since 1.22, however the macro only starts
+ * complaining in 1.24. That's intentional, because in 1.22 the asynchronous
+ * alternative was not yet available.
  */
-#define _NM_DEPRECATED_SYNC_METHOD            /*NM_DEPRECATED_IN_1_22*/
-#define _NM_DEPRECATED_SYNC_WRITABLE_PROPERTY /*NM_DEPRECATED_IN_1_22*/
+#define _NM_DEPRECATED_SYNC_METHOD               NM_DEPRECATED_IN_1_24
+#define _NM_DEPRECATED_SYNC_WRITABLE_PROPERTY /* NM_DEPRECATED_IN_1_22 */
 
 #endif  /* NM_VERSION_H */
