@@ -89,7 +89,7 @@ _nml_dbus_log (NMLDBusLogLevel level,
 		break;
 	}
 
-	ts = nm_utils_clock_gettime_ns (CLOCK_BOOTTIME);
+	ts = nm_utils_clock_gettime_nsec (CLOCK_BOOTTIME);
 
 	g_printerr ("libnm-dbus: %s[%"G_GINT64_FORMAT".%05"G_GINT64_FORMAT"] %s\n",
 	            prefix,
@@ -694,6 +694,7 @@ const NMLDBusMetaIface *const _nml_dbus_meta_ifaces[] = {
 	&_nml_dbus_meta_iface_nm_device_tun,
 	&_nml_dbus_meta_iface_nm_device_veth,
 	&_nml_dbus_meta_iface_nm_device_vlan,
+	&_nml_dbus_meta_iface_nm_device_vrf,
 	&_nml_dbus_meta_iface_nm_device_vxlan,
 	&_nml_dbus_meta_iface_nm_device_wifip2p,
 	&_nml_dbus_meta_iface_nm_device_wireguard,
@@ -844,6 +845,7 @@ nm_utils_g_param_spec_is_default (const GParamSpec *pspec)
 		return ((((GParamSpecString *) pspec)->default_value) == NULL);
 	if (NM_IN_SET (pspec->value_type, G_TYPE_BYTES,
 	                                  G_TYPE_PTR_ARRAY,
+	                                  G_TYPE_ARRAY,
 	                                  G_TYPE_HASH_TABLE,
 	                                  G_TYPE_STRV)) {
 		/* boxed types have NULL default. */
